# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for the cloud deploy release commands."""

import datetime
import enum
import os.path
import shutil
import tarfile
import uuid
from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.cloudbuild import snapshot
from googlecloudsdk.api_lib.clouddeploy import client_util
from googlecloudsdk.api_lib.clouddeploy import delivery_pipeline
from googlecloudsdk.api_lib.storage import storage_api
from googlecloudsdk.calliope import exceptions as c_exceptions
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.deploy import deploy_util
from googlecloudsdk.command_lib.deploy import exceptions
from googlecloudsdk.command_lib.deploy import manifest_util
from googlecloudsdk.command_lib.deploy import rollout_util
from googlecloudsdk.command_lib.deploy import skaffold_util
from googlecloudsdk.command_lib.deploy import staging_bucket_util
from googlecloudsdk.command_lib.deploy import target_util
from googlecloudsdk.command_lib.storage import errors as storage_errors
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
from googlecloudsdk.core import yaml
from googlecloudsdk.core.resource import resource_transform
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import times
from googlecloudsdk.generated_clients.apis.clouddeploy.v1 import clouddeploy_v1_messages as cd_messages
import six


_RELEASE_COLLECTION = (
    'clouddeploy.projects.locations.deliveryPipelines.releases'
)
_ALLOWED_SOURCE_EXT = ['.zip', '.tgz', '.gz']
_SOURCE_STAGING_TEMPLATE = 'gs://{}/source'
RESOURCE_NOT_FOUND = (
    'The following resources are snapped in the release, '
    'but no longer exist:\n{}\n\nThese resources were cached '
    'when the release was created, but their source '
    'may have been deleted.\n\n'
)
RESOURCE_CREATED = (
    'The following target is not snapped in the release:\n{}\n\n'
    "You may have specified a target that wasn't "
    'cached when the release was created.\n\n'
)
RESOURCE_CHANGED = (
    'The following snapped releases resources differ from '
    'their current definition:\n{}\n\nThe pipeline or targets '
    'were cached when the release was created, but the source '
    'has changed since then. You should review the differences '
    'before proceeding.\n'
)
_DATE_PATTERN = '$DATE'
_TIME_PATTERN = '$TIME'


GENERATED_SKAFFOLD = 'skaffold.yaml'


class Tools(enum.Enum):
  DOCKER = 'docker'
  HELM = 'helm'
  KPT = 'kpt'
  KUBECTL = 'kubectl'
  KUSTOMIZE = 'kustomize'
  SKAFFOLD = 'skaffold'


class ConfigCheckMode(enum.Enum):
  """The mode for configuration checking."""

  NONE = 'none'
  SKAFFOLD = 'skaffold'
  SKAFFOLD_AND_NATIVE = 'skaffold_and_native'
  NATIVE = 'native'


def RenderPattern(release_id):
  """Finds and replaces keywords in the release name.

    When adding to the list of keywords that can be expanded, care must be taken
    when two words share the same prefix ie. ($D and $DATE). In that case the
    longer keyword ($DATE) must be processed before the shorter one ($D).
  Args:
    release_id: str, the release name template.

  Returns:
    The formatted release name
  """
  time_now = datetime.datetime.now(datetime.timezone.utc)
  formatted_id = release_id.replace(_DATE_PATTERN, time_now.strftime('%Y%m%d'))
  formatted_id = formatted_id.replace(_TIME_PATTERN, time_now.strftime('%H%M'))
  _CheckForRemainingDollars(formatted_id)
  return formatted_id


def _CheckForRemainingDollars(release_id):
  """Find and notify user about dollar signs in release name."""

  dollar_positions = []
  for i in range(len(release_id)):
    if release_id[i] == '$':
      dollar_positions.append(six.text_type(i))
  if dollar_positions:
    raise exceptions.InvalidReleaseNameError(release_id, dollar_positions)


def SetBuildArtifacts(images, messages, release_config):
  """Set build_artifacts field of the release message.

  Args:
    images: dict[str,dict], docker image name and tag dictionary.
    messages: Module containing the Cloud Deploy messages.
    release_config: apitools.base.protorpclite.messages.Message, Cloud Deploy
      release message.

  Returns:
    Cloud Deploy release message.
  """
  if not images:
    return release_config
  build_artifacts = []
  for key, value in sorted(six.iteritems(images)):  # Sort for tests
    build_artifacts.append(messages.BuildArtifact(image=key, tag=value))
  release_config.buildArtifacts = build_artifacts

  return release_config


def LoadBuildArtifactFile(path):
  """Load images from a file containing JSON build data.

  Args:
    path: str, build artifacts file path.

  Returns:
    Docker image name and tag dictionary.
  """
  with files.FileReader(path) as f:  # Returns user-friendly error messages
    try:
      structured_data = yaml.load(f, file_hint=path)
    except yaml.Error as e:
      raise exceptions.ParserError(path, e.inner_error) from e
    images = {}
    for build in structured_data['builds']:
      # For b/191063894. Supporting both name for now.
      images[build.get('image', build.get('imageName'))] = build['tag']

    return images


def CreateReleaseConfig(
    *,  # go/pytype-strict-function-args
    source: str | None,
    gcs_source_staging_dir: str | None,
    ignore_file: str | None,
    images: dict[str, str],
    build_artifacts: str | None,
    description: str | None,
    docker_version: str | None,
    helm_version: str | None,
    kpt_version: str | None,
    kubectl_version: str | None,
    kustomize_version: str | None,
    skaffold_version: str | None,
    skaffold_file: str | None,
    location: str,
    pipeline_uuid: str,
    from_k8s_manifest: str | None,
    from_run_manifest: str | None,
    pipeline_obj: cd_messages.DeliveryPipeline,
    deploy_parameters: dict[str, str] | None,
    config_file: str | None = None,
    native_config_used: bool = False,
    native_source_specified: bool = False,
    hide_logs: bool = False,
):
  """Returns a build config."""
  messages = client_util.GetMessagesModule(client_util.GetClientInstance())
  release_config = messages.Release(description=description)
  if native_config_used:
    release_config = _ParseReleaseConfig(config_file, messages)
    release_config.description = description
    # If none of the native source flags are specified, then use the source
    # in the release config.
    if (
        not native_source_specified
        and release_config.source
        and release_config.source.storageSource
    ):
      source = _GcsUriFromStorageSource(release_config.source.storageSource)
      release_config.source = None

  _StageSourceObjectToGcs(
      release_config=release_config,
      source=source,
      gcs_source_staging_dir=gcs_source_staging_dir,
      ignore_file=ignore_file,
      location=location,
      pipeline_uuid=pipeline_uuid,
      kubernetes_manifest=from_k8s_manifest,
      cloud_run_manifest=from_run_manifest,
      skaffold_file=skaffold_file,
      pipeline_obj=pipeline_obj,
      native_config_used=native_config_used,
      hide_logs=hide_logs,
  )

  _SetVersion(
      release_config,
      messages,
      docker_version,
      helm_version,
      kpt_version,
      kubectl_version,
      kustomize_version,
      skaffold_version,
  )
  _SetImages(messages, release_config, images, build_artifacts)
  _SetDeployParameters(
      messages,
      deploy_util.ResourceType.RELEASE,
      release_config,
      deploy_parameters,
  )
  return release_config


def _CreateAndUploadTarball(
    gcs_client,
    gcs_source_staging,
    source,
    ignore_file,
    hide_logs,
):
  """Creates a local tarball and uploads it to GCS.

     After creating and uploading the tarball, this sets the Skaffold config URI
     in the release config.

  Args:
    gcs_client: client for Google Cloud Storage API.
    gcs_source_staging: directory in Google cloud storage to use for staging
    source: the location of the source files
    ignore_file: the ignore file to use
    hide_logs: whether to show logs, defaults to False

  Returns:
    the gcs uri where the tarball was uploaded.
  """
  source_snapshot = snapshot.Snapshot(source, ignore_file=ignore_file)
  size_str = resource_transform.TransformSize(source_snapshot.uncompressed_size)
  if not hide_logs:
    log.status.Print(
        'Creating temporary archive of %s file(s)'
        ' totalling %s before compression.',
        len(source_snapshot.files),
        size_str,
    )
  # This makes a tarball of the snapshot and then copies to GCS.
  staged_source_obj = source_snapshot.CopyArchiveToGCS(
      gcs_client,
      gcs_source_staging,
      ignore_file=ignore_file,
      hide_logs=hide_logs,
  )
  gcs_uri = f'gs://{staged_source_obj.bucket}/{staged_source_obj.name}#{staged_source_obj.generation}'
  return gcs_uri


def _SetVersion(
    release_config,
    messages,
    docker_version,
    helm_version,
    kpt_version,
    kubectl_version,
    kustomize_version,
    skaffold_version,
):
  """Set the version for the release config.

  Sets the ToolVersions for the release config or the SkaffoldVersion for the
  release config.

  The ToolVersions are always used if any of the tool version fields are set:
    docker_version
    helm_version
    kpt_version
    kubectl_version
    kustomize_version
  The ToolVersion of skaffold_version is only used if and only if the specified
  version is a full semver or 'latest'.

  The SkaffoldVersion on the release config is set if and only if
  skaffold_version is the only version specified and it does not match the
  full semver or 'latest'. This is purposefully done to allow uses to continue
  referencing existing supported Cloud Deploy images: e.g. 2.14/2.16.

  Args:
    release_config: a Release message
    messages: Module containing the Cloud Deploy messages.
    docker_version: the docker version to use, can be None.
    helm_version: the helm version to use, can be None.
    kpt_version: the kpt version to use, can be None.
    kubectl_version: the kubectl version to use, can be None.
    kustomize_version: the kustomize version to use, can be None.
    skaffold_version: the skaffold version to use, can be None.

  Returns:
    Modified release_config
  """
  # None and empty strings are handled in this manner.
  should_default = (
      not docker_version
      and not helm_version
      and not kpt_version
      and not kubectl_version
      and not kustomize_version
      and not skaffold_version
  )
  if should_default:
    return release_config
  # Skaffold is a different case because we want to allow users that specify
  # 2.14/2.16 to continue being able to do so until the image is expired.
  should_skaffold_use_tool_version = skaffold_version == 'latest' or (
      skaffold_version and skaffold_version.count('.') == 2
  )
  use_tool_version = (
      docker_version
      or helm_version
      or kpt_version
      or kubectl_version
      or kustomize_version
      or should_skaffold_use_tool_version
  )
  if not use_tool_version:
    release_config.skaffoldVersion = skaffold_version
    return release_config
  tool_versions = messages.ToolVersions(
      docker=docker_version,
      helm=helm_version,
      kpt=kpt_version,
      kubectl=kubectl_version,
      kustomize=kustomize_version,
      skaffold=skaffold_version,
  )
  release_config.toolVersions = tool_versions
  return release_config


def GetActualSource(
    args: parser_extensions.Namespace,
) -> tuple[str, bool]:
  """Determines the actual source location.

  Args:
    args: the argparse namespace

  Returns:
    A tuple of the actual source location and a boolean indicating if the
    native source flags are specified.
  """
  if args.IsKnownAndSpecified('local_source'):
    return args.local_source, True
  if args.IsKnownAndSpecified('gcs_source'):
    return args.gcs_source, True
  if args.IsKnownAndSpecified('source'):
    return args.source, False
  return os.getcwd(), False


def GetConfigCheckModeFromArgs(
    args: parser_extensions.Namespace,
):
  """Determines the config check mode."""
  if args.IsKnownAndSpecified('from_run_manifest') or args.IsKnownAndSpecified(
      'from_k8s_manifest'
  ):
    return ConfigCheckMode.NONE
  if (
      args.IsKnownAndSpecified('local_source')
      or args.IsKnownAndSpecified('gcs_source')
      or args.IsKnownAndSpecified('config_file')
  ):
    return ConfigCheckMode.NATIVE
  if not args.IsKnownAndSpecified('source'):
    # If no source is specified, check for skaffold.yaml, followed by
    # release.yaml. This implies that we might need to check both files.
    return ConfigCheckMode.SKAFFOLD_AND_NATIVE
  return ConfigCheckMode.SKAFFOLD


def VerifyConfigs(
    source: str,
    config_check_mode: ConfigCheckMode,
    skaffold_file: str | None,
    config_file: str | None,
) -> bool:
  """Verifies that the required configuration files exist in the source.

  Args:
    source: The actual source location.
    config_check_mode: The config check mode.
    skaffold_file: The skaffold file.
    config_file: The config file.

  Returns:
    A boolean indicating if the config file for native mode is used.
  """
  if config_check_mode is ConfigCheckMode.NONE:
    return False
  if config_check_mode is ConfigCheckMode.NATIVE:
    _VerifyReleaseConfigFileExists(config_file)
    return True
  if config_check_mode is ConfigCheckMode.SKAFFOLD:
    _VerifyFileExistsInSource(source, skaffold_file or 'skaffold.yaml')
    return False
  if config_check_mode is ConfigCheckMode.SKAFFOLD_AND_NATIVE:
    try:
      _VerifyFileExistsInSource(source, skaffold_file or 'skaffold.yaml')
      return False
    except c_exceptions.BadFileException:
      _VerifyReleaseConfigFileExists(config_file)
      return True
  return False


def _StageGcsObject(
    *,
    gcs_source_staging_dir: str | None,
    ignore_file: str | None,
    location: str,
    pipeline_uuid: str,
    actual_source: str,
    kubernetes_manifest: str | None,
    cloud_run_manifest: str | None,
    pipeline_obj: cd_messages.DeliveryPipeline,
    hide_logs: bool = False,
) -> str:
  """Processes GCS source and uploads to GCS.

  Args:
    gcs_source_staging_dir: directory in Google cloud storage to use for
      staging.
    ignore_file: the ignore file to use.
    location: the cloud region for the release.
    pipeline_uuid: the unique id of the release's parent pipeline.
    actual_source: the location of the source files.
    kubernetes_manifest: path to kubernetes manifest (e.g. /home/user/k8.yaml).
      If provided, a Skaffold file will be generated and uploaded to GCS on
      behalf of the customer.
    cloud_run_manifest: path to Cloud Run manifest (e.g.
      /home/user/service.yaml). If provided, a Skaffold file will be generated
      and uploaded to GCS on behalf of the customer.
    pipeline_obj: the pipeline_obj used for this release.
    hide_logs: whether to show logs, defaults to False.

  Returns:
    The gcs uri where the source was uploaded.
  """
  default_gcs_source = False
  default_bucket_name = staging_bucket_util.GetDefaultStagingBucket(
      pipeline_uuid
  )

  if gcs_source_staging_dir is None:
    default_gcs_source = True
    gcs_source_staging_dir = _SOURCE_STAGING_TEMPLATE.format(
        default_bucket_name
    )

  if not gcs_source_staging_dir.startswith('gs://'):
    raise c_exceptions.InvalidArgumentException(
        parameter_name='--gcs-source-staging-dir',
        message=gcs_source_staging_dir,
    )

  gcs_client = storage_api.StorageClient()
  suffix = '.tgz'

  if actual_source.startswith('gs://') or os.path.isfile(actual_source):
    # Determine type of the source by checking extensions.
    _, suffix = os.path.splitext(actual_source)

  # Next, stage the source to Cloud Storage.
  staged_object = (
      f'{times.GetTimeStampFromDateTime(times.Now())}-'
      f'{uuid.uuid4().hex}{suffix}'
  )
  gcs_source_staging_dir = resources.REGISTRY.Parse(
      gcs_source_staging_dir, collection='storage.objects'
  )

  try:
    gcs_client.CreateBucketIfNotExists(
        gcs_source_staging_dir.bucket,
        location=location,
        check_ownership=default_gcs_source,
        enable_uniform_level_access=True,
        enable_public_access_prevention=True,
    )
  except storage_api.BucketInWrongProjectError as e:
    # If we're using the default bucket but it already exists in a different
    # project, then it could belong to a malicious attacker (b/33046325).
    raise c_exceptions.RequiredArgumentException(
        'gcs-source-staging-dir',
        f'A bucket with name {default_bucket_name} already exists and is owned '
        'by another project. Specify a bucket using '
        '--gcs-source-staging-dir.',
    ) from e

  if gcs_source_staging_dir.object:
    staged_object = gcs_source_staging_dir.object + '/' + staged_object
  gcs_source_staging = resources.REGISTRY.Create(
      collection='storage.objects',
      bucket=gcs_source_staging_dir.bucket,
      object=staged_object,
  )

  gcs_uri = ''
  # If the user specified a gcs object and a gcs source staging, we need to
  # copy the object from the source to the staging dir.
  if actual_source.startswith('gs://'):
    gcs_obj = resources.REGISTRY.Parse(
        actual_source, collection='storage.objects'
    )
    staged_source_obj = gcs_client.Rewrite(gcs_obj, gcs_source_staging)
    gcs_uri = f'gs://{staged_source_obj.bucket}/{staged_source_obj.name}#{staged_source_obj.generation}'
  else:
    # If a Skaffold file should be generated
    if kubernetes_manifest or cloud_run_manifest:
      gcs_uri = _UploadTarballGeneratedSkaffoldAndManifest(
          kubernetes_manifest,
          cloud_run_manifest,
          gcs_client,
          gcs_source_staging,
          ignore_file,
          hide_logs,
          pipeline_obj,
      )
    elif os.path.isdir(actual_source):
      gcs_uri = _CreateAndUploadTarball(
          gcs_client,
          gcs_source_staging,
          actual_source,
          ignore_file,
          hide_logs,
      )
    # When its a tar file
    elif os.path.isfile(actual_source):
      if not hide_logs:
        log.status.Print(
            'Uploading local file [%s] to [gs://%s/%s].',
            actual_source,
            gcs_source_staging.bucket,
            gcs_source_staging.object,
        )
      staged_source_obj = gcs_client.CopyFileToGCS(
          actual_source, gcs_source_staging
      )
      gcs_uri = f'gs://{staged_source_obj.bucket}/{staged_source_obj.name}#{staged_source_obj.generation}'
  return gcs_uri


def _StageSourceObjectToGcs(
    *,
    release_config: cd_messages.Release,
    source: str | None,
    gcs_source_staging_dir: str | None,
    ignore_file: str | None,
    location: str,
    pipeline_uuid: str,
    kubernetes_manifest: str | None,
    cloud_run_manifest: str | None,
    skaffold_file: str | None,
    pipeline_obj: cd_messages.DeliveryPipeline,
    native_config_used: bool,
    hide_logs: bool = False,
) -> cd_messages.Release:
  """Processes the source for the release config.

  Creates a default Cloud Storage bucket with location for staging if
  gcs-source-staging-dir is not specified.

  Args:
    release_config: a Release message
    source: the location of the source files
    gcs_source_staging_dir: directory in google cloud storage to use for staging
    ignore_file: the ignore file to use
    location: the cloud region for the release
    pipeline_uuid: the unique id of the release's parent pipeline.
    kubernetes_manifest: path to kubernetes manifest (e.g. /home/user/k8.yaml).
      If provided, a Skaffold file will be generated and uploaded to GCS on
      behalf of the customer.
    cloud_run_manifest: path to Cloud Run manifest (e.g.
      /home/user/service.yaml).If provided, a Skaffold file will be generated
      and uploaded to GCS on behalf of the customer.
    skaffold_file: path of the skaffold file relative to the source directory
      that contains the Skaffold file.
    pipeline_obj: the pipeline_obj used for this release.
    native_config_used: whether native config is used.
    hide_logs: whether to show logs, defaults to False

  Returns:
    Modified release_config
  """
  gcs_uri = _StageGcsObject(
      gcs_source_staging_dir=gcs_source_staging_dir,
      ignore_file=ignore_file,
      location=location,
      pipeline_uuid=pipeline_uuid,
      actual_source=source,
      kubernetes_manifest=kubernetes_manifest,
      cloud_run_manifest=cloud_run_manifest,
      pipeline_obj=pipeline_obj,
      hide_logs=hide_logs,
  )
  messages = client_util.GetMessagesModule(client_util.GetClientInstance())
  if native_config_used:
    _SetStorageSource(release_config, messages, gcs_uri)
  else:
    skaffold_is_generated = bool(kubernetes_manifest or cloud_run_manifest)
    release_config = _SetSkaffoldConfigPath(
        release_config, skaffold_file, skaffold_is_generated
    )
    release_config.skaffoldConfigUri = gcs_uri

  return release_config


def _GetProfileToTargetMapping(pipeline_obj):
  """Gets mapping of profile to targets where the profile is activated."""
  profile_to_targets = {}
  for stage in pipeline_obj.serialPipeline.stages:
    for profile in stage.profiles:
      if profile not in profile_to_targets:
        profile_to_targets[profile] = []
      profile_to_targets[profile].append(stage.targetId)
  return profile_to_targets


def _GetUniqueProfilesToTargetMapping(profile_to_targets):
  """Gets mapping of profile to target only activated in a single target."""
  target_to_unique_profile = {}
  for profile, targets in profile_to_targets.items():
    if len(targets) == 1:
      target_to_unique_profile[targets[0]] = profile
  return target_to_unique_profile


def _GetTargetAndUniqueProfiles(pipeline_obj):
  """Get one unique profile for every target if it exists.

  Args:
    pipeline_obj: The Delivery Pipeline object.

  Returns:
    A map of target_id to profile.

  Raises:
   Error: If the pipeline targets don't each have a dedicated profile.
  """
  profile_to_targets = _GetProfileToTargetMapping(pipeline_obj)
  target_to_unique_profile = _GetUniqueProfilesToTargetMapping(
      profile_to_targets
  )

  # Every target should have one unique profile.
  if len(target_to_unique_profile) != len(pipeline_obj.serialPipeline.stages):
    raise core_exceptions.Error(
        'Target should use one profile not shared with another target.'
    )
  return target_to_unique_profile


def _UploadTarballGeneratedSkaffoldAndManifest(
    kubernetes_manifest,
    cloud_run_manifest,
    gcs_client,
    gcs_source_staging,
    ignore_file,
    hide_logs,
    pipeline_obj,
):
  """Generates a Skaffold file and uploads the file and k8 manifest to GCS.

  Args:
    kubernetes_manifest: path to kubernetes manifest (e.g. /home/user/k8.yaml).
      If provided, a Skaffold file will be generated and uploaded to GCS on
      behalf of the customer.
    cloud_run_manifest: path to Cloud Run manifest (e.g.
      /home/user/service.yaml). If provided, a Skaffold file will be generated
      and uploaded to GCS on behalf of the customer.
    gcs_client: client for Google Cloud Storage API.
    gcs_source_staging: directory in google cloud storage to use for staging
    ignore_file: the ignore file to use
    hide_logs: whether to show logs, defaults to False
    pipeline_obj: the pipeline_obj used for this release.

  Returns:
    the gcs uri where the tarball was uploaded.
  """
  with files.TemporaryDirectory() as temp_dir:
    manifest = ''
    skaffold_yaml = ''
    if kubernetes_manifest:
      manifest = kubernetes_manifest
      skaffold_yaml = skaffold_util.CreateSkaffoldFileForManifest(
          pipeline_obj,
          os.path.basename(manifest),
          skaffold_util.GKE_GENERATED_SKAFFOLD_TEMPLATE,
      )
    elif cloud_run_manifest:
      manifest = cloud_run_manifest
      skaffold_yaml = skaffold_util.CreateSkaffoldFileForManifest(
          pipeline_obj,
          os.path.basename(manifest),
          skaffold_util.CLOUD_RUN_GENERATED_SKAFFOLD_TEMPLATE,
      )
    # Check that the manifest file exists.
    if not os.path.exists(manifest):
      raise c_exceptions.BadFileException(
          f'could not find manifest file [{manifest}]'
      )
    # Create the YAML data. Copying to a temp directory to avoid editing
    # the local directory.
    shutil.copy(manifest, temp_dir)

    skaffold_path = os.path.join(temp_dir, GENERATED_SKAFFOLD)
    with files.FileWriter(skaffold_path) as f:
      # Prepend the auto-generated line to the YAML file
      f.write('# Auto-generated by Google Cloud Deploy\n')
      # Dump the yaml data to the Skaffold file.
      yaml.dump(skaffold_yaml, f, round_trip=True)
    gcs_uri = _CreateAndUploadTarball(
        gcs_client,
        gcs_source_staging,
        temp_dir,
        ignore_file,
        hide_logs,
    )
    log.status.Print(
        'Generated Skaffold file can be found here: {gcs_uri}'.format(
            gcs_uri=gcs_uri,
        )
    )
    return gcs_uri


def _VerifyFileExistsInSource(source: str, file_path: str | None):
  """Checks that the specified source contains the file.

  Args:
    source: the location of the source files
    file_path: path of the file relative to the source directory

  Raises:
    BadFileException: If the source directory or files can't be found.
  """
  if not file_path:
    raise c_exceptions.BadFileException(
        f'config file in source [{source}] is not specified'
    )

  if source.startswith('gs://'):
    log.status.Print(
        f'Skipping file check for [{file_path}]. '
        'Reason: source is not a local archive or directory'
    )
  elif not os.path.exists(source):
    raise c_exceptions.BadFileException(f'could not find source [{source}]')
  elif os.path.isfile(source):
    _VerifyFileIsInArchive(source, file_path)
  else:
    _VerifyFileIsInFolder(source, file_path)


def _VerifyReleaseConfigFileExists(file_path: str):
  """Checks if the release config file exists.

  This is different from the _VerifyFileExistsInSource function in that it
  will only verify the release config file without checking the source.
  The release config file is required for creating a release.

  Args:
    file_path: the path of the release config file.

  Raises:
    BadFileException: If the release config file can't be found.
  """

  if not file_path:
    raise c_exceptions.BadFileException(
        'release config file path is not specified.'
    )
  try:
    with files.FileReader(file_path):
      # Successfully opened the file, so it exists and is readable.
      pass
  except Exception as e:
    raise c_exceptions.BadFileException(
        f'could not access release config file [{file_path}]: {e}'
    )


def _VerifyFileIsInArchive(source: str, file_path: str):
  """Verifies the config file is in the archive.

  Args:
    source: the location of the source archive.
    file_path: path of the file in the source archive.

  Raises:
    BadFileException: If the config file is not a readable compressed file or
    can't be found.
  """
  _, ext = os.path.splitext(source)
  if ext not in _ALLOWED_SOURCE_EXT:
    raise c_exceptions.BadFileException(
        'local file [{src}] is none of ' + ', '.join(_ALLOWED_SOURCE_EXT)
    )
  if not tarfile.is_tarfile(source):
    raise c_exceptions.BadFileException(
        'Specified source file is not a readable compressed file archive'
    )
  with tarfile.open(source, mode='r:gz') as archive:
    try:
      archive.getmember(file_path)
    except KeyError as e:
      raise c_exceptions.BadFileException(
          f'Could not find file [{file_path}]. File '
          'does not exist in source archive'
      ) from e


def _VerifyFileIsInFolder(source: str, file_path: str):
  """Verifies the config file is in the folder.

  Args:
    source: the location of the source files
    file_path: path of the file relative to the source directory

  Raises:
    BadFileException: If the config file can't be found.
  """
  path_to_file = os.path.join(source, file_path)
  if not os.path.exists(path_to_file):
    raise c_exceptions.BadFileException(
        f'Could not find file [{path_to_file}]. File does not exist'
    )


def _SetImages(messages, release_config, images, build_artifacts):
  """Set the image substitutions for the release config."""
  if build_artifacts:
    images = LoadBuildArtifactFile(build_artifacts)

  return SetBuildArtifacts(images, messages, release_config)


def _SetSkaffoldConfigPath(release_config, skaffold_file, is_generated):
  """Sets the path for skaffold configuration file relative to source dir."""
  if skaffold_file:
    release_config.skaffoldConfigPath = skaffold_file
  if is_generated:
    release_config.skaffoldConfigPath = GENERATED_SKAFFOLD

  return release_config


def _SetDeployParameters(
    messages, resource_type, release_config, deploy_parameters
):
  """Set the deploy parameters for the release config."""
  if deploy_parameters:
    dps_value_msg = getattr(messages, resource_type.value).DeployParametersValue
    dps_value = dps_value_msg()
    for key, value in deploy_parameters.items():
      dps_value.additionalProperties.append(
          dps_value_msg.AdditionalProperty(key=key, value=value)
      )

    release_config.deployParameters = dps_value
  return release_config


def ListCurrentDeployedTargets(release_ref, targets):
  """Lists the targets where the given release is the latest.

  Args:
    release_ref: protorpc.messages.Message, protorpc.messages.Message, release
      reference.
    targets: protorpc.messages.Message, protorpc.messages.Message, list of
      target objects.

  Returns:
    A list of target references where this release is deployed.
  """
  matching_targets = []
  release_name = release_ref.RelativeName()
  pipeline_ref = release_ref.Parent()
  for obj in targets:
    target_name = obj.name
    target_ref = target_util.TargetReferenceFromName(target_name)
    # Gets the latest rollout of this target
    rollout_obj = target_util.GetCurrentRollout(target_ref, pipeline_ref)
    if rollout_obj is None:
      continue
    rollout_ref = rollout_util.RolloutReferenceFromName(rollout_obj.name)
    deployed_release_name = rollout_ref.Parent().RelativeName()
    if release_name == deployed_release_name:
      matching_targets.append(target_ref)
  return matching_targets


def DiffSnappedPipeline(release_ref, release_obj, to_target=None):
  """Detects differences between current and snapped delivery pipelines.

  Changes are determined through etag value differences.

  This runs the following checks:
    - if the to_target is one of the snapped targets in the release.
    - if the snapped targets still exist.
    - if the snapped targets have been changed.
    - if the snapped pipeline still exists.
    - if the snapped pipeline has been changed.

  Args:
    release_ref: protorpc.messages.Message, release resource object.
    release_obj: apitools.base.protorpclite.messages.Message, release message.
    to_target: str, the target to promote the release to. If specified, this
      verifies if the target has been snapped in the release.

  Returns:
    the list of the resources that no longer exist.
    the list of the resources that have been changed.
    the list of the resources that aren't snapped in the release.
  """
  resource_not_found = []
  resource_changed = []
  resource_created = []
  # check if the to_target is one of the snapped targets in the release.
  if to_target:
    ref_dict = release_ref.AsDict()
    # Creates shared target by default.
    target_ref = target_util.TargetReference(
        to_target,
        ref_dict['projectsId'],
        ref_dict['locationsId'],
    )
    # Only compare the resource ID, for the case that
    # if release_ref is parsed from arguments, it will use project ID,
    # whereas, the project number is stored in the DB.

    if target_ref.Name() not in [
        target_util.TargetId(obj.name) for obj in release_obj.targetSnapshots
    ]:
      resource_created.append(target_ref.RelativeName())

  for obj in release_obj.targetSnapshots:
    target_name = obj.name
    # Check if the snapped targets still exist.
    try:
      target_obj = target_util.GetTarget(
          target_util.TargetReferenceFromName(target_name)
      )
      # Checks if the snapped targets have been changed.
      if target_obj.etag != obj.etag:
        resource_changed.append(target_name)
    except apitools_exceptions.HttpError as error:
      log.debug('Failed to get target {}: {}'.format(target_name, error))
      log.status.Print('Unable to get target {}\n'.format(target_name))
      resource_not_found.append(target_name)

  name = release_obj.deliveryPipelineSnapshot.name
  # Checks if the pipeline exists.
  try:
    pipeline_obj = delivery_pipeline.DeliveryPipelinesClient().Get(name)
    # Checks if the pipeline has been changed.
    if pipeline_obj.etag != release_obj.deliveryPipelineSnapshot.etag:
      resource_changed.append(release_ref.Parent().RelativeName())
  except apitools_exceptions.HttpError as error:
    log.debug('Failed to get pipeline {}: {}'.format(name, error.content))
    log.status.Print('Unable to get delivery pipeline {}'.format(name))
    resource_not_found.append(name)

  return resource_created, resource_changed, resource_not_found


def PrintDiff(release_ref, release_obj, target_id=None, prompt=''):
  """Prints differences between current and snapped pipelines and targets.

  Args:
    release_ref: protorpc.messages.Message, release resource object.
    release_obj: apitools.base.protorpclite.messages.Message, release message.
    target_id: str, target id, e.g. test/stage/prod.
    prompt: str, prompt text.
  """
  resource_created, resource_changed, resource_not_found = DiffSnappedPipeline(
      release_ref, release_obj, target_id
  )

  if resource_created:
    prompt += RESOURCE_CREATED.format('\n'.join(BulletedList(resource_created)))
  if resource_not_found:
    prompt += RESOURCE_NOT_FOUND.format(
        '\n'.join(BulletedList(resource_not_found))
    )
  if resource_changed:
    prompt += RESOURCE_CHANGED.format('\n'.join(BulletedList(resource_changed)))

  log.status.Print(prompt)


def BulletedList(str_list):
  """Converts a list of string to a bulleted list.

  The returned list looks like ['- string1','- string2'].

  Args:
    str_list: [str], list to be converted.

  Returns:
    list of the transformed strings.
  """
  for i in range(len(str_list)):
    str_list[i] = '- ' + str_list[i]

  return str_list


def GetSnappedTarget(release_obj, target_id):
  """Get the snapped target in a release by target ID.

  Args:
    release_obj: apitools.base.protorpclite.messages.Message, release message
      object.
    target_id: str, target ID.

  Returns:
    target message object.
  """
  target_obj = None

  for ss in release_obj.targetSnapshots:
    if target_util.TargetId(ss.name) == target_id:
      target_obj = ss
      break

  return target_obj


def CheckReleaseSupportState(release_obj, action):
  """Checks the support state on a release.

  If the release is in maintenance mode, a warning will be logged.
  If the release is in expiration mode, an exception will be raised.

  Args:
    release_obj: The release object to check.
    action: the action that is being performed that requires the check.

  Raises: an core_exceptions.Error if any support state is unsupported
  """
  tools_in_maintenance = []
  tools_unsupported = []
  messages = client_util.GetMessagesModule(client_util.GetClientInstance())
  tools = [
      Tools.DOCKER,
      Tools.HELM,
      Tools.KPT,
      Tools.KUBECTL,
      Tools.KUSTOMIZE,
      Tools.SKAFFOLD,
  ]
  for t in tools:
    state = _GetToolVersionSupportState(release_obj, t)
    if not state:
      continue
    tool_version_enum = (
        messages.ToolVersionSupportedCondition.ToolVersionSupportStateValueValuesEnum
    )
    if state == tool_version_enum.TOOL_VERSION_SUPPORT_STATE_UNSUPPORTED:
      tools_unsupported.append(t)
    elif state == tool_version_enum.TOOL_VERSION_SUPPORT_STATE_MAINTENANCE_MODE:
      tools_in_maintenance.append(t)
    else:
      continue
  # A singular unsupported tool prevents a release from being supported.
  if tools_unsupported:
    joined = ', '.join([t.value for t in tools_unsupported])
    raise core_exceptions.Error(
        f"You can't {action} because the versions used for tools: [{joined}] "
        'are no longer supported.\n'
        'https://cloud.google.com/deploy/docs/select-tool-version'
    )
  if tools_in_maintenance:
    joined = ', '.join([t.value for t in tools_in_maintenance])
    log.status.Print(
        f'WARNING: The versions used for tools: [{joined}] are in maintenance '
        'mode and will be unsupported soon.\n'
        'https://cloud.google.com/deploy/docs/select-tool-version'
    )
    return
  # The old skaffold support state is correctly backfilled even if the release
  # uses tools.
  # This is mostly for releases that don't use tools.
  skaffold_support_state = _GetSkaffoldSupportState(release_obj)
  skaffold_support_state_enum = (
      messages.SkaffoldSupportedCondition.SkaffoldSupportStateValueValuesEnum
  )
  if (
      skaffold_support_state
      == skaffold_support_state_enum.SKAFFOLD_SUPPORT_STATE_UNSUPPORTED
  ):
    raise core_exceptions.Error(
        f"You can't {action} because the Skaffold version that was"
        ' used to create the release is no longer supported.\n'
        'https://cloud.google.com/deploy/docs/using-skaffold/select-skaffold'
        '#skaffold_version_deprecation_and_maintenance_policy'
    )

  if (
      skaffold_support_state
      == skaffold_support_state_enum.SKAFFOLD_SUPPORT_STATE_MAINTENANCE_MODE
  ):
    log.status.Print(
        "WARNING: This release's Skaffold version is in maintenance mode and"
        ' will be unsupported soon.\n'
        ' https://cloud.google.com/deploy/docs/using-skaffold/select-skaffold'
        '#skaffold_version_deprecation_and_maintenance_policy'
    )


def _GetSkaffoldSupportState(release_obj):
  """Gets the Skaffold Support State from the release.

  Args:
    release_obj: release message obj.

  Returns:
    None or SkaffoldSupportStateValueValuesEnum
  """
  # NOMUTANTS
  if release_obj.condition and release_obj.condition.skaffoldSupportedCondition:
    return release_obj.condition.skaffoldSupportedCondition.skaffoldSupportState
  return None


def _GetToolVersionSupportState(release_obj, tool):
  """Gets the Tool Version Support State from the release for a particular tool.

  Args:
    release_obj: release message obj.
    tool: Tools.Enum.

  Returns:
    None or ToolVersionSupportStateValueValuesEnum
  """
  if not release_obj.condition:
    return None
  if tool == Tools.DOCKER:
    if release_obj.condition.dockerVersionSupportedCondition:
      return (
          release_obj.condition.dockerVersionSupportedCondition.toolVersionSupportState
      )
  elif tool == Tools.HELM:
    if release_obj.condition.helmVersionSupportedCondition:
      return (
          release_obj.condition.helmVersionSupportedCondition.toolVersionSupportState
      )
  elif tool == Tools.KPT:
    if release_obj.condition.kptVersionSupportedCondition:
      return (
          release_obj.condition.kptVersionSupportedCondition.toolVersionSupportState
      )
  elif tool == Tools.KUBECTL:
    if release_obj.condition.kubectlVersionSupportedCondition:
      return (
          release_obj.condition.kubectlVersionSupportedCondition.toolVersionSupportState
      )
  elif tool == Tools.KUSTOMIZE:
    if release_obj.condition.kustomizeVersionSupportedCondition:
      return (
          release_obj.condition.kustomizeVersionSupportedCondition.toolVersionSupportState
      )
  elif tool == Tools.SKAFFOLD:
    if release_obj.condition.skaffoldVersionSupportedCondition:
      return (
          release_obj.condition.skaffoldVersionSupportedCondition.toolVersionSupportState
      )
  return None


def _SetStorageSource(
    release_config: cd_messages.Release, messages: cd_messages, gcs_uri: str
):
  """Sets the storageSource in the Release config from a GCS URI.

  Args:
    release_config: The Release config to set the storageSource in.
    messages: The messages module to use.
    gcs_uri: The GCS URI to set the storageSource to.

  Raises:
    googlecloudsdk.calliope.exceptions.InvalidArgumentException: If the
      provided GCS URI is invalid or not a GCS URI.
  """
  try:
    storage_object = storage_url.CloudUrl.from_url_string(gcs_uri)
  except storage_errors.InvalidUrlError as e:
    raise c_exceptions.InvalidArgumentException(
        parameter_name='staged gcs uri',
        message=f'invalid gcs uri format: {gcs_uri}',
    ) from e

  if storage_object.scheme is not storage_url.ProviderPrefix.GCS:
    raise c_exceptions.InvalidArgumentException(
        parameter_name='staged gcs uri',
        message=f'storageSource must start with "gs://", got {gcs_uri}',
    )
  storage_source = messages.GoogleCloudStorageSource(
      bucket=storage_object.bucket_name,
      object=storage_object.resource_name,
      generation=int(storage_object.generation),
  )
  if not release_config.source:
    release_config.source = messages.Source(storageSource=storage_source)
  else:
    release_config.source.storageSource = storage_source


def _GcsUriFromStorageSource(
    storage_source: cd_messages.GoogleCloudStorageSource,
) -> str:
  """Converts a source object to a GCS URI.

  Args:
    storage_source: The storageSource field from a Release source object.

  Returns:
    The GCS URI.
  """
  gcs_uri = f'gs://{storage_source.bucket}/{storage_source.object}'
  if storage_source.generation:
    gcs_uri += f'#{storage_source.generation}'
  return gcs_uri


def _ParseReleaseConfig(
    config_file: str, messages: cd_messages
) -> cd_messages.Release:
  """Parses TargetConfigs from a release config file.

  Args:
    config_file: The path to the release config file.
    messages: The messages module.

  Returns:
    The parsed TargetConfigs.

  Raises:
    c_exceptions.InvalidArgumentException: If the config file is invalid or
      cannot be parsed.
  """
  try:
    parsed_yaml = yaml.load_path(config_file)
  except yaml.Error as e:
    raise c_exceptions.InvalidArgumentException(
        parameter_name='--config-file',
        message=f'failed to parse config file: {e}',
    ) from e

  if parsed_yaml is None:
    raise c_exceptions.InvalidArgumentException(
        parameter_name='--config-file',
        message=f'config file [{config_file}] is empty',
    )
  return manifest_util.ParseReleaseConfig(parsed_yaml, messages)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a Cloud FTP user."""

from googlecloudsdk.api_lib.storage import ftp
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.storage.ftp import operations_util
from googlecloudsdk.command_lib.storage.ftp import servers_util
from googlecloudsdk.command_lib.storage.ftp import users_util
from googlecloudsdk.core import log


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Create(base.Command):
  """Create a new Cloud FTP user."""

  hints = base.CommandHint(read_only=False)

  detailed_help = {
      'DESCRIPTION': (
          """\
          Create a new Cloud FTP user for a specific server.
      """
      ),
      'EXAMPLES': (
          """\
          To create a user `my-user` for server `my-server` in `us-west1`, run:

            $ {command} my-user --server=my-server --location=us-west1 --customer-service-account=my-sa@my-project.iam.gserviceaccount.com --user-credentials-from-file=keys.json --storage-directory-mapping=bucket=my-bucket,directory=/home/user,permission=READ_WRITE
      """
      ),
  }

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'USER_ID',
        help='The ID of the user to create.',
    )
    parser.add_argument(
        '--async',
        action='store_true',
        dest='async_',
        help=(
            'Return immediately, without waiting for the operation to complete.'
        ),
    )
    parser.add_argument(
        '--server',
        required=True,
        help='The ID of the server.',
    )
    parser.add_argument(
        '--location',
        required=True,
        help='The region of the server.',
    )
    parser.add_argument(
        '--customer-service-account',
        required=True,
        help='The service account email associated with this user.',
    )
    parser.add_argument(
        '--user-credentials-from-file',
        type=arg_parsers.FileContents(),
        help='Path to a JSON file containing user credentials.',
    )
    parser.add_argument(
        '--storage-directory-mapping',
        type=arg_parsers.ArgDict(
            spec={
                'bucket': str,
                'bucket_prefix': str,
                'directory': str,
                'permission': str,
            },
            required_keys=['bucket', 'directory', 'permission'],
        ),
        action='append',
        metavar='PROPERTY=VALUE',
        help='Mapping of Google Cloud Storage bucket to virtual directory.',
    )

  def Run(self, args):
    client = ftp.FtpClient()
    parent = servers_util.GetServerResourceName(args.location, args.server)
    user_msg = users_util.CreateUserMsg(client.messages, args)

    op = client.CreateUser(parent, args.USER_ID, user_msg)

    if args.async_:
      log.status.Print('Create request issued for: [{}]'.format(args.USER_ID))
      log.status.Print('Check operation [{}] for status.'.format(op.name))
      return op

    op_ref = operations_util.GetOperationRef(op.name)
    return operations_util.WaitForOperation(
        op_ref,
        'Waiting for user [{}] to be created'.format(args.USER_ID),
        result_service=client.users_service,
    )



"""Generated message classes for lustre version v1alpha.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'lustre'


class AccessRule(_messages.Message):
  r"""A single policy group with IP-based access rules for the Managed Lustre
  instance.

  Enums:
    AccessModeValueValuesEnum: Optional. The access mode for the access rule
      nodemap. Default is READ_WRITE.
    SquashModeValueValuesEnum: Required. Squash mode for the access rule.

  Fields:
    accessMode: Optional. The access mode for the access rule nodemap. Default
      is READ_WRITE.
    ipAddressRanges: Required. The IP address ranges to which to apply this
      access rule. Accepts non-overlapping CIDR ranges (e.g.,
      `192.168.1.0/24`) and IP addresses (e.g., `192.168.1.0`).
    mountableSubdirectories: Optional. The list of non-root directories that
      can be mounted from clients in this NID range subset. Currently, there
      can be only a single directory at most. If no directory is mentioned,
      then the root directory will be accessible.
    name: Required. The name of the access rule policy group. Must be 16
      characters or less and include only alphanumeric characters or '_'.
    squashGid: Optional. Squash GID for the access rule. If the squash mode
      for this rule is ROOT_SQUASH, root users matching the ip_address_ranges
      are squashed to this GID. Defaults to 0 (no root squash).
    squashMode: Required. Squash mode for the access rule.
    squashUid: Optional. Squash UID for the access rule. If the squash mode
      for this rule is ROOT_SQUASH, root users matching the ip_address_ranges
      are squashed to this UID. Defaults to 0 (no root squash).
  """

  class AccessModeValueValuesEnum(_messages.Enum):
    r"""Optional. The access mode for the access rule nodemap. Default is
    READ_WRITE.

    Values:
      ACCESS_MODE_UNSPECIFIED: Unspecified access mode.
      READ_ONLY: Read-only access mode.
      READ_WRITE: Read-write access mode.
    """
    ACCESS_MODE_UNSPECIFIED = 0
    READ_ONLY = 1
    READ_WRITE = 2

  class SquashModeValueValuesEnum(_messages.Enum):
    r"""Required. Squash mode for the access rule.

    Values:
      SQUASH_MODE_UNSPECIFIED: Unspecified squash mode.
      NO_SQUASH: Squash is disabled. If set inside an AccessRule, root users
        matching the ip_ranges are not squashed. If set as the
        default_squash_mode, root squash is disabled for this instance. If the
        default squash mode is `NO_SQUASH`, do not set the default_squash_uid
        or default_squash_gid, or an `invalid argument` error is returned.
      ROOT_SQUASH: Root user squash is enabled. Not supported inside an
        AccessRule. If set as the default_squash_mode, root users not matching
        any of the access_rules are squashed to the default_squash_uid and
        default_squash_gid.
      ALL_USERS_SQUASH: All users squashed to the squash_uid and squash_gid
        for the access rule. If this is for the default_squash_mode, then the
        default_squash_uid and default_squash_gid will be squashed.
    """
    SQUASH_MODE_UNSPECIFIED = 0
    NO_SQUASH = 1
    ROOT_SQUASH = 2
    ALL_USERS_SQUASH = 3

  accessMode = _messages.EnumField('AccessModeValueValuesEnum', 1)
  ipAddressRanges = _messages.StringField(2, repeated=True)
  mountableSubdirectories = _messages.StringField(3, repeated=True)
  name = _messages.StringField(4)
  squashGid = _messages.IntegerField(5, variant=_messages.Variant.INT32)
  squashMode = _messages.EnumField('SquashModeValueValuesEnum', 6)
  squashUid = _messages.IntegerField(7, variant=_messages.Variant.INT32)


class AccessRulesOptions(_messages.Message):
  r"""IP-based access rules for the Managed Lustre instance. These options
  define the root user squash configuration.

  Enums:
    DefaultSquashModeValueValuesEnum: Required. The squash mode for the
      default access rule.

  Fields:
    accessRules: Optional. The access rules for the instance.
    defaultMountableSubdirectories: Optional. The list of non-root directories
      that can be mounted from clients in the default access rule. Currently,
      there can be only a single directory at most. If no directory is
      mentioned, then the root directory will be accessible.
    defaultSquashGid: Optional. The user squash GID for the default access
      rule. This user squash GID applies to all root users connecting from
      clients that are not matched by any of the access rules. If not set, the
      default is 0 (no GID squash).
    defaultSquashMode: Required. The squash mode for the default access rule.
    defaultSquashUid: Optional. The user squash UID for the default access
      rule. This user squash UID applies to all root users connecting from
      clients that are not matched by any of the access rules. If not set, the
      default is 0 (no UID squash).
  """

  class DefaultSquashModeValueValuesEnum(_messages.Enum):
    r"""Required. The squash mode for the default access rule.

    Values:
      SQUASH_MODE_UNSPECIFIED: Unspecified squash mode.
      NO_SQUASH: Squash is disabled. If set inside an AccessRule, root users
        matching the ip_ranges are not squashed. If set as the
        default_squash_mode, root squash is disabled for this instance. If the
        default squash mode is `NO_SQUASH`, do not set the default_squash_uid
        or default_squash_gid, or an `invalid argument` error is returned.
      ROOT_SQUASH: Root user squash is enabled. Not supported inside an
        AccessRule. If set as the default_squash_mode, root users not matching
        any of the access_rules are squashed to the default_squash_uid and
        default_squash_gid.
      ALL_USERS_SQUASH: All users squashed to the squash_uid and squash_gid
        for the access rule. If this is for the default_squash_mode, then the
        default_squash_uid and default_squash_gid will be squashed.
    """
    SQUASH_MODE_UNSPECIFIED = 0
    NO_SQUASH = 1
    ROOT_SQUASH = 2
    ALL_USERS_SQUASH = 3

  accessRules = _messages.MessageField('AccessRule', 1, repeated=True)
  defaultMountableSubdirectories = _messages.StringField(2, repeated=True)
  defaultSquashGid = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  defaultSquashMode = _messages.EnumField('DefaultSquashModeValueValuesEnum', 4)
  defaultSquashUid = _messages.IntegerField(5, variant=_messages.Variant.INT32)


class AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. * `domain:{domain}`: The G Suite domain
      (primary) that represents all the users of that domain. For example,
      `google.com` or `example.com`. * `principal://iam.googleapis.com/locatio
      ns/global/workforcePools/{pool_id}/subject/{subject_attribute_value}`: A
      single identity in a workforce identity pool. * `principalSet://iam.goog
      leapis.com/locations/global/workforcePools/{pool_id}/group/{group_id}`:
      All workforce identities in a group. * `principalSet://iam.googleapis.co
      m/locations/global/workforcePools/{pool_id}/attribute.{attribute_name}/{
      attribute_value}`: All workforce identities with a specific attribute
      value. * `principalSet://iam.googleapis.com/locations/global/workforcePo
      ols/{pool_id}/*`: All identities in a workforce identity pool. * `princi
      pal://iam.googleapis.com/projects/{project_number}/locations/global/work
      loadIdentityPools/{pool_id}/subject/{subject_attribute_value}`: A single
      identity in a workload identity pool. * `principalSet://iam.googleapis.c
      om/projects/{project_number}/locations/global/workloadIdentityPools/{poo
      l_id}/group/{group_id}`: A workload identity pool group. * `principalSet
      ://iam.googleapis.com/projects/{project_number}/locations/global/workloa
      dIdentityPools/{pool_id}/attribute.{attribute_name}/{attribute_value}`:
      All identities in a workload identity pool with a certain attribute. * `
      principalSet://iam.googleapis.com/projects/{project_number}/locations/gl
      obal/workloadIdentityPools/{pool_id}/*`: All identities in a workload
      identity pool. * `deleted:user:{emailid}?uid={uniqueid}`: An email
      address (plus unique identifier) representing a user that has been
      recently deleted. For example,
      `alice@example.com?uid=123456789012345678901`. If the user is recovered,
      this value reverts to `user:{emailid}` and the recovered user retains
      the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `deleted:principal://iam.google
      apis.com/locations/global/workforcePools/{pool_id}/subject/{subject_attr
      ibute_value}`: Deleted single identity in a workforce identity pool. For
      example, `deleted:principal://iam.googleapis.com/locations/global/workfo
      rcePools/my-pool-id/subject/my-subject-attribute-value`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`. For an
      overview of the IAM roles and permissions, see the [IAM
      documentation](https://cloud.google.com/iam/docs/roles-overview). For a
      list of the available pre-defined roles, see
      [here](https://cloud.google.com/iam/docs/understanding-roles).
  """

  condition = _messages.MessageField('Expr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class Date(_messages.Message):
  r"""Represents a whole or partial calendar date, such as a birthday. The
  time of day and time zone are either specified elsewhere or are
  insignificant. The date is relative to the Gregorian Calendar. This can
  represent one of the following: * A full date, with non-zero year, month,
  and day values. * A month and day, with a zero year (for example, an
  anniversary). * A year on its own, with a zero month and a zero day. * A
  year and month, with a zero day (for example, a credit card expiration
  date). Related types: * google.type.TimeOfDay * google.type.DateTime *
  google.protobuf.Timestamp

  Fields:
    day: Day of a month. Must be from 1 to 31 and valid for the year and
      month, or 0 to specify a year by itself or a year and month where the
      day isn't significant.
    month: Month of a year. Must be from 1 to 12, or 0 to specify a year
      without a month and day.
    year: Year of the date. Must be from 1 to 9999, or 0 to specify a date
      without a year.
  """

  day = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  month = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  year = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class DirectoryPolicy(_messages.Message):
  r"""A directory policy for a Managed Lustre instance.

  Fields:
    directoryPath: Required. Immutable. The lustre instance filesystem full
      path of the directory. It must start with a slash. e.g.
      /lustre/testFolder
    lustreProjectId: Output only. The lustre project ID assigned for the
      directory by the service. This read-only ID can be used to manage
      quotas. See more details in https://docs.cloud.google.com/managed-
      lustre/docs/file-system-quotas#set_quotas
    name: Immutable. Identifier. The resource name of the directory policy.
      DirectoryPolicy names have the form `projects/{project}/locations/{locat
      ion}/instances/{instance}/directoryPolicies/{id}`. {id} is user
      provided.
    uid: Output only. Unique ID of the resource.
  """

  directoryPath = _messages.StringField(1)
  lustreProjectId = _messages.IntegerField(2)
  name = _messages.StringField(3)
  uid = _messages.StringField(4)


class DynamicTierOptions(_messages.Message):
  r"""Dynamic tier options for a Managed Lustre instance.

  Enums:
    ModeValueValuesEnum: Required. Immutable. The dynamic tier mode of the
      instance.

  Fields:
    mode: Required. Immutable. The dynamic tier mode of the instance.
  """

  class ModeValueValuesEnum(_messages.Enum):
    r"""Required. Immutable. The dynamic tier mode of the instance.

    Values:
      MODE_UNSPECIFIED: Unspecified dynamic tier mode.
      DISABLED: The dynamic tier is explicitly disabled.
      DEFAULT_CACHE: The dynamic tier is enabled.
    """
    MODE_UNSPECIFIED = 0
    DISABLED = 1
    DEFAULT_CACHE = 2

  mode = _messages.EnumField('ModeValueValuesEnum', 1)


class ExportDataRequest(_messages.Message):
  r"""Export data from Managed Lustre to a Cloud Storage bucket.

  Fields:
    gcsPath: The URI to a Cloud Storage bucket, or a path within a bucket,
      using the format `gs:////`. If a path inside the bucket is specified, it
      must end with a forward slash (`/`).
    lustrePath: The root directory path to the Managed Lustre file system.
      Must start with `/`. Default is `/`.
    requestId: Optional. UUID to identify requests.
    serviceAccount: Optional. User-specified service account used to perform
      the transfer. If unspecified, the Managed Lustre service agent is used.
      Use one of the following formats: * `{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/{PROJECT_ID}/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/-/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}`
  """

  gcsPath = _messages.MessageField('GcsPath', 1)
  lustrePath = _messages.MessageField('LustrePath', 2)
  requestId = _messages.StringField(3)
  serviceAccount = _messages.StringField(4)


class Expr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class GcsPath(_messages.Message):
  r"""Specifies a Cloud Storage bucket and, optionally, a path inside the
  bucket.

  Fields:
    uri: Required. The URI to a Cloud Storage bucket, or a path within a
      bucket, using the format `gs:////`. If a path inside the bucket is
      specified, it must end with a forward slash (`/`).
  """

  uri = _messages.StringField(1)


class GoogleProtobufEmpty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class ImportDataRequest(_messages.Message):
  r"""Message for importing data to Lustre.

  Fields:
    gcsPath: The Cloud Storage source bucket and, optionally, path inside the
      bucket. If a path inside the bucket is specified, it must end with a
      forward slash (`/`).
    lustrePath: Lustre path destination.
    requestId: Optional. UUID to identify requests.
    serviceAccount: Optional. User-specified service account used to perform
      the transfer. If unspecified, the default Managed Lustre service agent
      will be used. Use one of the following formats: *
      `{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/{PROJECT_ID}/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}` *
      `projects/-/serviceAccounts/{EMAIL_ADDRESS_OR_UNIQUE_ID}`
  """

  gcsPath = _messages.MessageField('GcsPath', 1)
  lustrePath = _messages.MessageField('LustrePath', 2)
  requestId = _messages.StringField(3)
  serviceAccount = _messages.StringField(4)


class Instance(_messages.Message):
  r"""A Managed Lustre instance.

  Enums:
    AuditLogLevelValueValuesEnum: Optional. The file system audit log level
      for the instance.
    InitialCompressionAlgorithmValueValuesEnum: Optional. Input only. The
      compression algorithm to use when initializing the instance.
    StateValueValuesEnum: Output only. The state of the instance.

  Messages:
    LabelsValue: Optional. Labels as key value pairs.

  Fields:
    accessRulesOptions: Optional. The access rules options for the instance.
    auditLogLevel: Optional. The file system audit log level for the instance.
    capacityGib: Required. The storage capacity of the instance in gibibytes
      (GiB). Allowed values depend on the `perUnitStorageThroughput`. See
      [Performance tiers](https://docs.cloud.google.com/managed-
      lustre/docs/performance-tiers) for specific minimums, maximums, and step
      sizes for each performance tier.
    createTime: Output only. Timestamp when the instance was created.
    description: Optional. A user-readable description of the instance.
    dynamicTierOptions: Optional. Immutable. Specifies whether the instance is
      on the Dynamic tier. See [Performance
      tiers](https://docs.cloud.google.com/managed-lustre/docs/performance-
      tiers) for more information.
    filesystem: Required. Immutable. The filesystem name for this instance.
      This name is used by client-side tools, including when mounting the
      instance. Must be eight characters or less and can only contain letters
      and numbers.
    gkeSupportEnabled: Optional. Deprecated: No longer required for GKE
      instance creation. Indicates whether you want to enable support for GKE
      clients. By default, GKE clients are not supported.
    iamAccessControlEnabled: Optional. Whether the IAM-based access control is
      enabled or not.
    initialCompressionAlgorithm: Optional. Input only. The compression
      algorithm to use when initializing the instance.
    initialCompressionLevel: Optional. Input only. The compression level to
      use for when initializing the instance. Defaults to 1 if not set. Valid
      ranges depend on the selected algorithm: - LZ4: 1 to 12 - ZSTD: 1 to 22
      Higher levels generally offer better compression ratios but may increase
      CPU usage.
    kmsKey: Optional. Immutable. The Cloud KMS key name to use for data
      encryption. If not set, the instance will use Google-managed encryption
      keys. If set, the instance will use customer-managed encryption keys.
      The key must be in the same region as the instance. The key format is: p
      rojects/{project}/locations/{location}/keyRings/{key_ring}/cryptoKeys/{k
      ey}
    labels: Optional. Labels as key value pairs.
    maintenancePolicy: Optional. The maintenance policy for the instance to
      determine when to allow or exclude the instance from maintenance
      updates.
    mountPoint: Output only. Mount point of the instance in the format
      `IP_ADDRESS@tcp:/FILESYSTEM`.
    name: Identifier. The name of the instance.
    network: Required. Immutable. The full name of the VPC network to which
      the instance is connected. Must be in the format
      `projects/{project_id}/global/networks/{network_name}`.
    perUnitStorageThroughput: Optional. The throughput of the instance in MBps
      per TiB. Valid values are 0, 125, 250, 500, 1000. See [Performance
      tiers](https://docs.cloud.google.com/managed-lustre/docs/performance-
      tiers) for more information. If the instance is using the Dynamic tier,
      this field must not be set or must be set to zero.
    placementPolicy: Optional. The placement policy name for the instance in
      the format of projects/{project}/locations/{location}/resourcePolicies/{
      resource_policy}
    satisfiesPzi: Output only. Reserved for future use
    satisfiesPzs: Output only. Reserved for future use
    state: Output only. The state of the instance.
    stateReason: Output only. The reason why the instance is in a certain
      state (e.g. SUSPENDED).
    uid: Output only. Unique ID of the resource. This is unrelated to the
      access rules which allow specifying the root squash uid.
    upcomingMaintenanceSchedule: Output only. Date and time of upcoming
      maintenance for the instance, if a maintenance policy is set.
    updateTime: Output only. Timestamp when the instance was last updated.
  """

  class AuditLogLevelValueValuesEnum(_messages.Enum):
    r"""Optional. The file system audit log level for the instance.

    Values:
      AUDIT_LOG_LEVEL_UNSPECIFIED: Not set.
      DISABLED: File system audit logs are explicitly disabled.
      BASIC: File system basic audit logs are enabled.
    """
    AUDIT_LOG_LEVEL_UNSPECIFIED = 0
    DISABLED = 1
    BASIC = 2

  class InitialCompressionAlgorithmValueValuesEnum(_messages.Enum):
    r"""Optional. Input only. The compression algorithm to use when
    initializing the instance.

    Values:
      COMPRESSION_ALGORITHM_UNSPECIFIED: Not set.
      LZ4: Compression algorithm is explicitly enabled with LZ4 compression.
      ZSTD: Compression algorithm is explicitly enabled with ZSTD compression.
    """
    COMPRESSION_ALGORITHM_UNSPECIFIED = 0
    LZ4 = 1
    ZSTD = 2

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the instance.

    Values:
      STATE_UNSPECIFIED: Not set.
      ACTIVE: The instance is available for use.
      CREATING: The instance is being created and is not yet ready for use.
      DELETING: The instance is being deleted.
      UPGRADING: The instance is being upgraded.
      REPAIRING: The instance is being repaired.
      STOPPED: The instance is stopped.
      UPDATING: The instance is being updated.
      SUSPENDED: The instance is suspended due to an issue related to Cloud
        KMS. The details are available in state_reason.
      BACKING_UP: The instance is being backed up.
      MAINTENANCE: The instance is under maintenance.
    """
    STATE_UNSPECIFIED = 0
    ACTIVE = 1
    CREATING = 2
    DELETING = 3
    UPGRADING = 4
    REPAIRING = 5
    STOPPED = 6
    UPDATING = 7
    SUSPENDED = 8
    BACKING_UP = 9
    MAINTENANCE = 10

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Optional. Labels as key value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  accessRulesOptions = _messages.MessageField('AccessRulesOptions', 1)
  auditLogLevel = _messages.EnumField('AuditLogLevelValueValuesEnum', 2)
  capacityGib = _messages.IntegerField(3)
  createTime = _messages.StringField(4)
  description = _messages.StringField(5)
  dynamicTierOptions = _messages.MessageField('DynamicTierOptions', 6)
  filesystem = _messages.StringField(7)
  gkeSupportEnabled = _messages.BooleanField(8)
  iamAccessControlEnabled = _messages.BooleanField(9)
  initialCompressionAlgorithm = _messages.EnumField('InitialCompressionAlgorithmValueValuesEnum', 10)
  initialCompressionLevel = _messages.IntegerField(11, variant=_messages.Variant.INT32)
  kmsKey = _messages.StringField(12)
  labels = _messages.MessageField('LabelsValue', 13)
  maintenancePolicy = _messages.MessageField('MaintenancePolicy', 14)
  mountPoint = _messages.StringField(15)
  name = _messages.StringField(16)
  network = _messages.StringField(17)
  perUnitStorageThroughput = _messages.IntegerField(18)
  placementPolicy = _messages.StringField(19)
  satisfiesPzi = _messages.BooleanField(20)
  satisfiesPzs = _messages.BooleanField(21)
  state = _messages.EnumField('StateValueValuesEnum', 22)
  stateReason = _messages.StringField(23)
  uid = _messages.StringField(24)
  upcomingMaintenanceSchedule = _messages.MessageField('MaintenanceSchedule', 25)
  updateTime = _messages.StringField(26)


class ListDirectoryPoliciesResponse(_messages.Message):
  r"""Response message for ListDirectoryPolicies.

  Fields:
    directoryPolicies: The list of DirectoryPolicies.
    nextPageToken: A token identifying a page of results the server should
      return.
  """

  directoryPolicies = _messages.MessageField('DirectoryPolicy', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListInstancesResponse(_messages.Message):
  r"""Message for response to listing Instances

  Fields:
    instances: Response from ListInstances.
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Unordered list. Locations that could not be reached.
  """

  instances = _messages.MessageField('Instance', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LustrePath(_messages.Message):
  r"""The root directory path to the Lustre file system.

  Fields:
    path: Optional. The root directory path to the Managed Lustre file system.
      Must start with `/`. Default is `/`. If you're importing data into
      Managed Lustre, any path other than the default must already exist on
      the file system.
  """

  path = _messages.StringField(1)


class LustreProjectsLocationsGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesCreateRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesCreateRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    instanceId: Required. The name of the Managed Lustre instance. * Must
      contain only lowercase letters, numbers, and hyphens. * Must start with
      a letter. * Must be between 1-63 characters. * Must end with a number or
      a letter.
    parent: Required. The instance's project and location, in the format
      `projects/{project}/locations/{location}`. Locations map to Google Cloud
      zones; for example, `us-west1-b`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  instance = _messages.MessageField('Instance', 1)
  instanceId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class LustreProjectsLocationsInstancesDeleteRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDeleteRequest object.

  Fields:
    force: Optional. If set to true, any sub-resources from this instance will
      also be deleted. Otherwise, the request will only work if the instance
      has no sub-resources.
    name: Required. The resource name of the instance to delete, in the format
      `projects/{projectId}/locations/{location}/instances/{instanceId}`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)


class LustreProjectsLocationsInstancesDirectoryPoliciesCreateRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesCreateRequest object.

  Fields:
    directoryPolicy: A DirectoryPolicy resource to be passed as the request
      body.
    directoryPolicyId: Required. The ID for the DirectoryPolicy to create.
    parent: Required. The parent instance. It must be in the format of
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  directoryPolicy = _messages.MessageField('DirectoryPolicy', 1)
  directoryPolicyId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesDeleteRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesDeleteRequest object.

  Fields:
    name: Required. The resource name of the directory policy. DirectoryPolicy
      names have the form `projects/{project}/locations/{location}/instances/{
      instance}/directoryPolicies/{id}`.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest
  object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesGetRequest object.

  Fields:
    name: Required. The resource name of the directory policy. DirectoryPolicy
      names have the form `projects/{project}/locations/{location}/instances/{
      instance}/directoryPolicies/{id}`.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesListRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesListRequest object.

  Fields:
    pageSize: Optional. Requested page size. Server might return fewer items
      than requested. If unspecified, the server will pick an appropriate
      default. The maximum value is 1000; values above 1000 will be coerced to
      1000.
    pageToken: Optional. A page token, received from a previous
      `ListDirectoryPolicies` call. Provide this to retrieve the subsequent
      page. When paginating, all other parameters provided to
      `ListDirectoryPolicies` must match the call that provided the page
      token.
    parent: Required. The parent instance. It must be in the format of
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class LustreProjectsLocationsInstancesDirectoryPoliciesSetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesDirectoryPoliciesSetIamPolicyRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class LustreProjectsLocationsInstancesDirectoryPoliciesTestIamPermissionsRequest(_messages.Message):
  r"""A
  LustreProjectsLocationsInstancesDirectoryPoliciesTestIamPermissionsRequest
  object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class LustreProjectsLocationsInstancesExportDataRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesExportDataRequest object.

  Fields:
    exportDataRequest: A ExportDataRequest resource to be passed as the
      request body.
    name: Required. The name of the Managed Lustre instance in the format
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  exportDataRequest = _messages.MessageField('ExportDataRequest', 1)
  name = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesGetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesGetIamPolicyRequest object.

  Fields:
    options_requestedPolicyVersion: Optional. The maximum policy version that
      will be used to format the policy. Valid values are 0, 1, and 3.
      Requests specifying an invalid value will be rejected. Requests for
      policies with any conditional role bindings must specify version 3.
      Policies with no conditional role bindings may specify any valid value
      or leave the field unset. The policy in the response might use the
      policy version that you specified, or it might use a lower policy
      version. For example, if you specify version 3, but the policy has no
      conditional role bindings, the response uses version 1. To learn which
      resources support conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    resource: REQUIRED: The resource for which the policy is being requested.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
  """

  options_requestedPolicyVersion = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  resource = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesGetRequest object.

  Fields:
    name: Required. The instance resource name, in the format
      `projects/{projectId}/locations/{location}/instances/{instanceId}`.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsInstancesImportDataRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesImportDataRequest object.

  Fields:
    importDataRequest: A ImportDataRequest resource to be passed as the
      request body.
    name: Required. The name of the Managed Lustre instance in the format
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  importDataRequest = _messages.MessageField('ImportDataRequest', 1)
  name = _messages.StringField(2, required=True)


class LustreProjectsLocationsInstancesListRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesListRequest object.

  Fields:
    filter: Optional. Filtering results.
    orderBy: Optional. Desired order of results.
    pageSize: Optional. Requested page size. Server might return fewer items
      than requested. If unspecified, the server will pick an appropriate
      default.
    pageToken: Optional. A token identifying a page of results the server
      should return.
    parent: Required. The project and location for which to retrieve a list of
      instances, in the format `projects/{projectId}/locations/{location}`. To
      retrieve instance information for all locations, use "-" as the value of
      `{location}`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class LustreProjectsLocationsInstancesPatchRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesPatchRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    name: Identifier. The name of the instance.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Optional. Specifies the fields to be overwritten in the
      instance resource by the update. The fields specified in the update_mask
      are relative to the resource, not the full request. A field will be
      overwritten if it is in the mask. If no mask is provided then all fields
      present in the request are overwritten.
  """

  instance = _messages.MessageField('Instance', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class LustreProjectsLocationsInstancesRescheduleMaintenanceRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesRescheduleMaintenanceRequest object.

  Fields:
    name: Required. Format:
      projects/{project}/locations/{location}/instances/{instance}
    rescheduleMaintenanceRequest: A RescheduleMaintenanceRequest resource to
      be passed as the request body.
  """

  name = _messages.StringField(1, required=True)
  rescheduleMaintenanceRequest = _messages.MessageField('RescheduleMaintenanceRequest', 2)


class LustreProjectsLocationsInstancesSetIamPolicyRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesSetIamPolicyRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy is being specified.
      See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    setIamPolicyRequest: A SetIamPolicyRequest resource to be passed as the
      request body.
  """

  resource = _messages.StringField(1, required=True)
  setIamPolicyRequest = _messages.MessageField('SetIamPolicyRequest', 2)


class LustreProjectsLocationsInstancesTestIamPermissionsRequest(_messages.Message):
  r"""A LustreProjectsLocationsInstancesTestIamPermissionsRequest object.

  Fields:
    resource: REQUIRED: The resource for which the policy detail is being
      requested. See [Resource
      names](https://cloud.google.com/apis/design/resource_names) for the
      appropriate value for this field.
    testIamPermissionsRequest: A TestIamPermissionsRequest resource to be
      passed as the request body.
  """

  resource = _messages.StringField(1, required=True)
  testIamPermissionsRequest = _messages.MessageField('TestIamPermissionsRequest', 2)


class LustreProjectsLocationsListRequest(_messages.Message):
  r"""A LustreProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class LustreProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class LustreProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class LustreProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A LustreProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class MaintenanceExclusionWindow(_messages.Message):
  r"""Exclusion period when maintenance updates should not occur. An exclusion
  window can be in either of the following two formats: * Non-recurring : A
  full date, with non-zero year, month and day values. * Recurring : A month
  and day value, with a zero year. Time zone is UTC.

  Fields:
    endDate: Required. End date of the exclusion period in UTC time zone. This
      date is inclusive.
    startDate: Required. Start date of the exclusion period in UTC time zone.
      This date is inclusive.
    time: Required. Time in UTC when the exclusion window starts on start_date
      and ends on end_date. This can be: * Full time OR * All zeros for
      00:00:00 UTC
  """

  endDate = _messages.MessageField('Date', 1)
  startDate = _messages.MessageField('Date', 2)
  time = _messages.MessageField('TimeOfDay', 3)


class MaintenancePolicy(_messages.Message):
  r"""Defines a maintenance policy for a resource.

  Fields:
    maintenanceExclusionWindow: Optional. The exclusion windows for the
      instance. Currently limited to 1 window.
    weeklyMaintenanceWindows: Required. The weekly maintenance windows for the
      instance. Currently limited to 1 window.
  """

  maintenanceExclusionWindow = _messages.MessageField('MaintenanceExclusionWindow', 1, repeated=True)
  weeklyMaintenanceWindows = _messages.MessageField('WeeklyMaintenanceWindow', 2, repeated=True)


class MaintenanceSchedule(_messages.Message):
  r"""Represents a scheduled maintenance event.

  Fields:
    endTime: Output only. The scheduled end time for the maintenance.
    startTime: Output only. The scheduled start time for the maintenance.
  """

  endTime = _messages.StringField(1)
  startTime = _messages.StringField(2)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of a long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** ``` { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: -
  members: - user:mike@example.com - group:admins@example.com -
  domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com
  role: roles/resourcemanager.organizationAdmin - members: -
  user:eve@example.com role: roles/resourcemanager.organizationViewer
  condition: title: expirable access description: Does not grant access after
  Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
  etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features,
  see the [IAM documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class ReconciliationOperationMetadata(_messages.Message):
  r"""Operation metadata returned by the CLH during resource state
  reconciliation.

  Enums:
    ExclusiveActionValueValuesEnum: Excluisive action returned by the CLH.

  Fields:
    deleteResource: DEPRECATED. Use exclusive_action instead.
    exclusiveAction: Excluisive action returned by the CLH.
  """

  class ExclusiveActionValueValuesEnum(_messages.Enum):
    r"""Excluisive action returned by the CLH.

    Values:
      UNKNOWN_REPAIR_ACTION: Unknown repair action.
      DELETE: The resource has to be deleted. When using this bit, the CLH
        should fail the operation. DEPRECATED. Instead use DELETE_RESOURCE
        OperationSignal in SideChannel.
      RETRY: This resource could not be repaired but the repair should be
        tried again at a later time. This can happen if there is a dependency
        that needs to be resolved first- e.g. if a parent resource must be
        repaired before a child resource.
    """
    UNKNOWN_REPAIR_ACTION = 0
    DELETE = 1
    RETRY = 2

  deleteResource = _messages.BooleanField(1)
  exclusiveAction = _messages.EnumField('ExclusiveActionValueValuesEnum', 2)


class Reschedule(_messages.Message):
  r"""The desired reschedule settings.

  Enums:
    RescheduleTypeValueValuesEnum: Required. The type of rescheduling.

  Fields:
    rescheduleType: Required. The type of rescheduling.
    scheduleTime: Optional. Required if reschedule_type is BY_TIME. Timestamp
      when the maintenance shall be rescheduled to. This time must be within
      28 days of the original scheduled maintenance start time.
  """

  class RescheduleTypeValueValuesEnum(_messages.Enum):
    r"""Required. The type of rescheduling.

    Values:
      RESCHEDULE_TYPE_UNSPECIFIED: Unspecified schedule type.
      IMMEDIATE: Apply update immediately
      NEXT_AVAILABLE_WINDOW: Reschedule to the next available window.
      BY_TIME: Reschedule to a specific time.
    """
    RESCHEDULE_TYPE_UNSPECIFIED = 0
    IMMEDIATE = 1
    NEXT_AVAILABLE_WINDOW = 2
    BY_TIME = 3

  rescheduleType = _messages.EnumField('RescheduleTypeValueValuesEnum', 1)
  scheduleTime = _messages.StringField(2)


class RescheduleMaintenanceRequest(_messages.Message):
  r"""Message for requesting to reschedule a maintenance event for a specific
  instance.

  Fields:
    requestId: Optional. A unique identifier for this request. A random UUID
      is recommended. This request is only idempotent if a `request_id` is
      provided.
    reschedule: Required. The desired reschedule settings.
  """

  requestId = _messages.StringField(1)
  reschedule = _messages.MessageField('Reschedule', 2)


class SetIamPolicyRequest(_messages.Message):
  r"""Request message for `SetIamPolicy` method.

  Fields:
    policy: REQUIRED: The complete policy to be applied to the `resource`. The
      size of the policy is limited to a few 10s of KB. An empty policy is a
      valid policy but certain Google Cloud services (such as Projects) might
      reject them.
    updateMask: OPTIONAL: A FieldMask specifying which fields of the policy to
      modify. Only the fields in the mask will be modified. If no mask is
      provided, the following default mask is used: `paths: "bindings, etag"`
  """

  policy = _messages.MessageField('Policy', 1)
  updateMask = _messages.StringField(2)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TestIamPermissionsRequest(_messages.Message):
  r"""Request message for `TestIamPermissions` method.

  Fields:
    permissions: The set of permissions to check for the `resource`.
      Permissions with wildcards (such as `*` or `storage.*`) are not allowed.
      For more information see [IAM
      Overview](https://cloud.google.com/iam/docs/overview#permissions).
  """

  permissions = _messages.StringField(1, repeated=True)


class TestIamPermissionsResponse(_messages.Message):
  r"""Response message for `TestIamPermissions` method.

  Fields:
    permissions: A subset of `TestPermissionsRequest.permissions` that the
      caller is allowed.
  """

  permissions = _messages.StringField(1, repeated=True)


class TimeOfDay(_messages.Message):
  r"""Represents a time of day. The date and time zone are either not
  significant or are specified elsewhere. An API may choose to allow leap
  seconds. Related types are google.type.Date and `google.protobuf.Timestamp`.

  Fields:
    hours: Hours of a day in 24 hour format. Must be greater than or equal to
      0 and typically must be less than or equal to 23. An API may choose to
      allow the value "24:00:00" for scenarios like business closing time.
    minutes: Minutes of an hour. Must be greater than or equal to 0 and less
      than or equal to 59.
    nanos: Fractions of seconds, in nanoseconds. Must be greater than or equal
      to 0 and less than or equal to 999,999,999.
    seconds: Seconds of a minute. Must be greater than or equal to 0 and
      typically must be less than or equal to 59. An API may allow the value
      60 if it allows leap-seconds.
  """

  hours = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  minutes = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  nanos = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  seconds = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class WeeklyMaintenanceWindow(_messages.Message):
  r"""Weekly time window in which maintenance updates may occur. Duration of
  the window is currently fixed at 1 hour. Time zone is UTC.

  Enums:
    DayOfWeekValueValuesEnum: Required. Day of the week for the maintenance
      window.

  Fields:
    dayOfWeek: Required. Day of the week for the maintenance window.
    startTime: Required. Start time of the maintenance window in UTC time
      zone.
  """

  class DayOfWeekValueValuesEnum(_messages.Enum):
    r"""Required. Day of the week for the maintenance window.

    Values:
      DAY_OF_WEEK_UNSPECIFIED: The day of the week is unspecified.
      MONDAY: Monday
      TUESDAY: Tuesday
      WEDNESDAY: Wednesday
      THURSDAY: Thursday
      FRIDAY: Friday
      SATURDAY: Saturday
      SUNDAY: Sunday
    """
    DAY_OF_WEEK_UNSPECIFIED = 0
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7

  dayOfWeek = _messages.EnumField('DayOfWeekValueValuesEnum', 1)
  startTime = _messages.MessageField('TimeOfDay', 2)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
encoding.AddCustomJsonFieldMapping(
    LustreProjectsLocationsInstancesGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')
encoding.AddCustomJsonFieldMapping(
    LustreProjectsLocationsInstancesDirectoryPoliciesGetIamPolicyRequest, 'options_requestedPolicyVersion', 'options.requestedPolicyVersion')
